
# q-expansion of newform 225.2.h.b, downloaded from the LMFDB on 09 November 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 225
weight = 2
poly_data = [1, -1, 1, -1, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = [[101, [1, 0, 0, 0]], [127, [0, 0, 0, -1]]]
aps_data = [[0, 1, -1, 0], [0, 0, 0, 0], [2, -1, 2, 0], [-1, 0, -1, 1], [-2, 4, -4, 2], [3, 0, 3, 0], [0, -2, 4, -2], [0, -3, -1, -3], [7, -5, 5, -7], [-1, 1, 0, -2], [0, 0, -3, 0], [-2, -1, -2, 0], [-2, -2, -2, 0], [0, 0, 3, -3], [1, -1, 0, -1], [-4, 4, 0, 3], [3, -9, 3, 0], [1, 5, -5, -1], [0, 2, 6, 2], [-1, 1, 0, -5], [9, -9, 9, -9], [5, -5, 0, 0], [0, -2, 5, -2], [4, 4, -4, -4], [3, -3, 0, -1], [5, 0, 4, -4], [-9, 9, 0, 6], [-3, 0, -12, 12], [0, -10, 0, 0], [-3, 15, -3, 0], [6, 10, -10, -6], [0, 11, 0, 11], [8, -7, 8, 0], [-5, 5, -5, 5], [-9, 0, -8, 8], [0, 0, -9, 9], [-3, 0, -10, 10], [0, -11, 0, 0], [0, 9, -9, 9], [-7, -9, 9, 7], [4, -4, 0, -7], [0, 6, 4, 6], [-10, -8, -10, 0], [-2, 0, -6, 6], [6, -6, 0, -6], [-3, 0, 9, -9], [-7, -3, 3, 7], [-16, 6, -6, 16], [18, -16, 16, -18], [-6, 6, 0, -12], [0, -8, -2, -8], [-27, 23, -23, 27], [-4, -5, -4, 0], [13, 0, -10, 10], [-18, 0, 3, -3], [-5, -14, -5, 0], [0, 2, 14, 2], [0, 0, 0, 8], [15, -21, 21, -15], [0, 5, -7, 5], [0, 3, -28, 3], [26, 0, 4, -4], [6, 0, -2, 2], [-19, 2, -2, 19], [2, 18, 2, 0], [0, 14, 1, 14], [0, 18, -12, 18], [3, -6, 3, 0], [5, -5, 0, -28], [12, 0, -6, 6], [-5, 5, 0, 21], [-19, 2, -19, 0], [0, 9, -20, 9], [4, -19, 19, -4], [-22, 22, 0, 21], [0, 20, -21, 20], [15, -15, 15, -15], [-13, 13, 0, -8], [9, 0, 22, -22], [-12, -9, -12, 0], [0, -4, 7, -4], [0, 0, 0, -32], [0, -24, 15, -24], [0, -3, 25, -3], [7, -28, 28, -7], [-17, 0, 8, -8], [-16, 0, -7, 7], [-2, 0, 12, -12], [17, -7, 7, -17], [-18, 13, -18, 0], [0, 25, -12, 25], [-3, 3, 0, -6], [-12, 29, -12, 0], [36, -21, 36, 0], [-7, 0, -9, 9], [-12, 12, 0, -18], [7, -16, 7, 0], [20, -20, 0, -3], [18, -15, 15, -18], [18, -16, 18, 0], [-6, 6, 0, 31], [-8, 0, -2, 2], [1, 8, 1, 0], [0, -4, -23, -4], [18, -18, 0, -21], [-18, -14, 14, 18], [6, 9, 6, 0], [19, 0, 5, -5], [3, 0, -4, 4], [27, 0, 15, -15], [-21, 0, 9, -9], [-21, 2, -21, 0], [0, 2, -13, 2], [0, 4, 33, 4], [0, 2, -9, 2], [5, 2, 5, 0], [8, 0, 24, -24], [-30, 30, 0, 12], [5, -5, 0, 11], [4, 18, 4, 0], [24, 3, -3, -24], [4, 6, -6, -4], [9, -10, 10, -9], [0, -4, 11, -4], [15, -27, 15, 0], [-14, 0, -34, 34], [-30, 15, -30, 0], [0, 6, 27, 6], [-10, 19, -19, 10], [0, 21, -7, 21], [18, 3, -3, -18], [4, 0, 20, -20], [-31, 0, -16, 16], [11, 0, -12, 12], [22, 3, -3, -22], [0, -12, 6, -12], [14, 17, -17, -14], [10, 18, 10, 0], [-2, 2, 0, 13], [-21, 18, -21, 0], [5, -11, 11, -5], [-27, 27, 0, 3], [-38, 44, -44, 38], [-22, 9, -9, 22], [3, -3, 0, 31], [6, -9, 9, -6], [28, -28, 0, -30], [24, 0, 27, -27], [12, 9, 12, 0], [-9, -27, -9, 0], [-12, -18, 18, 12], [0, -20, 28, -20], [0, -12, -28, -12], [-16, 20, -16, 0], [25, 0, 36, -36], [-37, 39, -39, 37], [0, 23, -39, 23], [35, -35, 0, -15], [-15, -27, -15, 0], [12, -39, 12, 0], [14, -14, 0, -20], [-19, 19, 0, 23], [0, 16, 14, 16], [1, -1, 0, 4], [-25, 11, -11, 25], [0, 1, -9, 1], [-20, 3, -20, 0], [-16, 16, 0, 1]]
